% HICP 4 Sector Breakdown .. read in prices and weights
% 11/5/2018

if load_data == 1;

% ----------- Sample Period, Calendars and so forth
[dnobs_m,calvec_m,calds_m] = calendar_make([2001 1],[2018 3],12);
[dnobs_q,calvec_q,calds_q] = calendar_make([2001 1],[2018 1],4);

% --------------- Read In Monthly Data for Nominal Price Indices ---------------- 
xlsname = '../data/HICPbreakdownEASANSA.xlsx';
% Read Data 
ndesc=1;
ncodes=0;
sheet='HICP items';
% Read Descriptors';
[~,descvec,~]=xlsread(xlsname,sheet,'B1:CP1');
descvec = descvec';

% Read Sector Numbers
[~,tmp,~]=xlsread(xlsname,sheet,'B2:CP2');
nn = size(tmp,2);
secnum = NaN(nn,1);
for i = 1:nn;
    aa = char(tmp(1,i));
    bb = strrep(aa,'_','9');
    cc = strrep(bb,'X','9');
    secnum(i) = str2num(cc);
end;
% Renumber Energy Sectors as Sector 13
elist = [045100;045200;045300;045400;045500;072200];
for i = 1:size(elist,1);
    ii = secnum == elist(i);
    tmp = elist(i)/10000;
    tmp = 10000*(13+tmp-floor(tmp));
    secnum(ii==1) = tmp;
end;
% Orgranize by Main Group
sec_cat = floor(secnum/10000);

% Read Price Indices
price_data=xlsread(xlsname,sheet,'B63:CP269');  % 2001:M1 = 2018:M3
p_disagg = price_data;

% Read Weights
sheet='Weights_items';
weight_data=xlsread(xlsname,sheet,'B63:CP279');  % 2001:M1 = 2018:M12
nyears = size(weight_data,1)/12;
if floor(nyears) ~= ceil(nyears);
    error('dataset doesnt have full years');
end;

% Aggregate weights into sectors
w_disagg_raw = weight_data;
% Interpolate annual weights to get monthly weights
w_disagg = NaN(size(w_disagg_raw));
Tgrid = (1:1:size(w_disagg_raw,1))';
for i = 1:size(w_disagg_raw,2);
    ydata = w_disagg_raw(:,i);
    y_interp = zeros(size(ydata,1),1);
    ifirst = 1;
    for j = 1:nyears;
       ii = (j-1)*12+1;
       if isnan(ydata(ii)) == 0;
           if ifirst == 1
               x = ydata(ii);
               ifirst = 0;
           else
               x = [x;ydata(ii)];
           end
       end
    end;
    tmp = interpolate_annual_to_monthly(x);
    w_disagg(:,i) = tmp;
%     plot(Tgrid,[w_disagg_raw(:,i) w_disagg(:,i)]);
%     waitforbuttonpress;
%     close(gcf);  
end;
w_disagg = w_disagg(1:size(p_disagg,1),:);

% Weights for Each of 13 Sector
w_sectors = NaN(dnobs_m,13);
p_sectors = NaN(dnobs_m,13);
for i = 1:13;
    % Select Prices and weights
    ii = sec_cat == i;
    p = p_disagg(:,ii==1);
    w = w_disagg(:,ii==1);
    w_sectors(:,i) = sum(w,2);
    w = w./repmat(sum(w,2),1,size(w,2));
    dp = dif(log(p),1);
    dp = sum(dp.*w,2);
    p_tmp = [0;cumsum(dp(2:end))];
    p_sectors(:,i) = 100*exp(p_tmp);
end;

namevec_sectors = { ...
    'Food' ...
    'Alcohol & Tobacco' ...
    'Clothing' ...
    'Housing' ...
    'Furnishings' ...
    'Healthcare' ...
    'Transportation' ...
    'Communications' ...
    'Recreation' ...
    'Education' ...
    'Restaurants & Accommodations' ...
    'Miscellaneous' ...
    'Energy' ...
    };

% normalize w_sectors to sum to 1;
w_sectors = w_sectors./repmat(sum(w_sectors,2),1,size(w_sectors,2));

% Construct aggregate
p_agg = zeros(dnobs_m,1);
dp = dif(log(p_sectors),1);
dp = sum(dp.*w_sectors,2);
p_agg(2:end) = cumsum(dp(2:end));
p_agg = 100*exp(p_agg);

% Construct Quarterly Averages of monthly series
p_agg = mtoq(p_agg,calds_m,calds_q,0);
p_disagg = mtoq(p_sectors,calds_m,calds_q,0);
w_disagg = mtoq(w_sectors,calds_m,calds_q,0);
calvec = calvec_q;
calds = calds_q;
dnobs = dnobs_q;
dp_disagg = 400*dif(log(p_disagg),1);
dp_agg = 400*dif(log(p_agg),1);
namevec_disagg = namevec_sectors;

% % Check .. read in aggregate monthly price data
% p_agg_1=xlsread('../data/HICPbreakdownEASANSA.xlsx','HICP main components','B63:B269');  % Through March 2018 (Q1)
% p_agg_1 = mtoq(p_agg_1,calds_m,calds_q,0);
% dp_agg_1 = 400*dif(log(p_agg_1),1);
% plot(calvec,[dp_agg dp_agg_1]);

 % Save Variable Series 
 slist13sectors = {...
           'calvec' ...
           'calds' ...
           'dnobs' ... 
           'dp_agg' ...
           'dp_disagg' ...
           'w_disagg' ...
           'namevec_disagg' ...
          }';
  str_tmp = [matdir 'slist13sectors'];
  save(str_tmp,'slist13sectors');
  for iseries = 1:size(slist13sectors,1);
     ustr = char(slist13sectors(iseries));
     str_tmp = [matdir ustr];
     str_tmp1 = 'save(str_tmp,''';
     str_tmp2 = ''');';
     eval([str_tmp1 ustr str_tmp2]);
  end;

end;  %Load Data end;
   
% Load Variables and Give Standard Names 
str_tmp = [matdir 'slist13sectors'];
load(str_tmp); 
for iseries = 1:size(slist13sectors,1);
     ustr = char(slist13sectors(iseries));
     str_tmp = [matdir ustr];
     load(str_tmp);
end; 
