% HICP 3 Sector Breakdown .. read in prices and weights
% 11/5/2018

if load_data == 1;

% ----------- Sample Period, Calendars and so forth
[dnobs_m,calvec_m,calds_m] = calendar_make([2001 1],[2018 3],12);
[dnobs_q,calvec_q,calds_q] = calendar_make([2001 1],[2018 1],4);

% --------------- Read In Monthly Data for Nominal Price Indices ---------------- 
xlsname = '../data/HICPbreakdownEASANSA.xlsx';
% Read Data 
ndesc=1;
ncodes=0;
sheet='HICP main components';
% Read Descriptors';
[~,descvec,~]=xlsread(xlsname,sheet,'B1:J1');
descvec = descvec';

% Read Price Indices
price_data=xlsread(xlsname,sheet,'B63:J269');  % 2001:M1 = 2018:M3

% Read Weights
sheet='Weights main components';  
weight_data=xlsread(xlsname,sheet,'B64:J279');  % 2001:M1 = 2018:M12
nyears = size(weight_data,1)/12;
if floor(nyears) ~= ceil(nyears);
    error('dataset doesnt have full years');
end;

% Extract Components
% Food, Ind. Goods-x-energy, Energy, Services
ii = [0 0 1 0 0 0 1 1 1]';
p_disagg = price_data(:,ii==1);
labelvec_disagg = descvec(ii==1);
namevec_disagg = { ...
    'Food' ...
    'Goods (x F&E)' ...
    'Energy' ...
    'Services' ...
    };
w_disagg_raw = weight_data(:,ii==1);
p_agg_1 = price_data(:,1);        % Aggregate Index

% Interpolate annual weights to get monthly weights
w_disagg = NaN(size(w_disagg_raw));
Tgrid = (1:1:size(w_disagg_raw,1))';
for i = 1:size(w_disagg_raw,2);
    ydata = w_disagg_raw(:,i);
    y_interp = zeros(size(ydata,1),1);
    ifirst = 1;
    for j = 1:nyears;
       ii = (j-1)*12+1;
       if ydata(ii) ~= 0;
           if ifirst == 1
               x = ydata(ii);
               ifirst = 0;
           else
               x = [x;ydata(ii)];
           end
       end
    end;
    tmp = interpolate_annual_to_monthly(x);
    ii = ydata ~= 0;
    w_disagg(ii==1,i) = tmp;
%     plot(Tgrid,[w_disagg_raw(:,i) w_disagg(:,i)]);
%     waitforbuttonpress;
%     close(gcf);  
end;
w_disagg = w_disagg(1:size(p_disagg,1),:);
% Compute Inflation Rates for each sector
dp_disagg = dif(log(p_disagg),1);

% Combine into 3 Sectors
w_disagg_4 = w_disagg;
dp_disagg_4 = dp_disagg;

w_disagg = NaN(size(w_disagg_4,1),3);
dp_disagg = NaN(size(dp_disagg_4,1),3);

namevec_disagg = { ...
    'HICP_xfe' ...
    'Food' ...
    'Energy' ...
    };
dp_disagg(:,2) = dp_disagg_4(:,1);
dp_disagg(:,3) = dp_disagg_4(:,3);
w_disagg(:,2) = w_disagg_4(:,1);
w_disagg(:,3) = w_disagg_4(:,3);
w_disagg(:,1) =  w_disagg_4(:,2) + w_disagg_4(:,4);
w_tmp = w_disagg_4(:,[2 4]);
w_tmp = w_tmp./repmat(sum(w_tmp,2),1,2);
dp_disagg(:,1) = sum(dp_disagg_4(:,[2 4]).*w_tmp,2);
tmp = (dp_disagg);
p_disagg = zeros(size(dp_disagg));
p_disagg(2:end,:) = cumsum(tmp(2:end,:));
p_disagg = 100*exp(p_disagg);


% Compute aggregate Index date-by-date
w_disagg = w_disagg./repmat(sum(w_disagg,2),1,size(w_disagg,2));
tmp = sum(dp_disagg.*w_disagg,2);
p_agg = zeros(dnobs_m,1);
p_agg(2:end) = cumsum(tmp(2:end));
p_agg = 100*exp(p_agg);

% Construct Quarterly Averages of monthly series
p_agg = mtoq(p_agg,calds_m,calds_q,0);
p_disagg = mtoq(p_disagg,calds_m,calds_q,0);
w_disagg = mtoq(w_disagg,calds_m,calds_q,0);
p_agg_1 = mtoq(p_agg_1,calds_m,calds_q,0);
calvec = calvec_q;
calds = calds_q;
dnobs = dnobs_q;
dp_disagg = 400*dif(log(p_disagg),1);
dp_agg = 400*dif(log(p_agg),1);
dp_agg_1 = 400*dif(log(p_agg_1),1);

% plot(calvec,[dp_agg dp_agg_1]);
% error('tmp');


 % Save Variable Series 
 slist3sectors = {...
           'calvec' ...
           'calds' ...
           'dnobs' ... 
           'dp_agg' ...
           'dp_disagg' ...
           'w_disagg' ...
           'labelvec_disagg' ...
           'namevec_disagg' ...
          }';
  str_tmp = [matdir 'slist3sectors'];
  save(str_tmp,'slist3sectors');
  for iseries = 1:size(slist3sectors,1);
     ustr = char(slist3sectors(iseries));
     str_tmp = [matdir ustr];
     str_tmp1 = 'save(str_tmp,''';
     str_tmp2 = ''');';
     eval([str_tmp1 ustr str_tmp2]);
  end;

end;  %Load Data end;
   
% Load Variables and Give Standard Names 
str_tmp = [matdir 'slist3sectors'];
load(str_tmp); 
for iseries = 1:size(slist3sectors,1);
     ustr = char(slist3sectors(iseries));
     str_tmp = [matdir ustr];
     load(str_tmp);
end; 
