function [tau_s,tau_f] = m_agg_tau_weight(y,alpha_eps,alpha_tau,alpha_s,sigma_eps_common,sigma_ds_common,sigma_dtau_common,sigma_eps_unique,sigma_ds_unique,sigma_dtau_unique,share)
      
small=1.0e-06;
big = 1.0e+6;
n_y = size(y,2);
nobs = size(y,1);

% Set up State Vector
  % --- State Vector
  %     (1) eps(t)
  %     (2) tau(t)
  %     (3) tau_u(t)
  %     (4) seas(t) (with 3 lags)
  %     (5) seas_u(t) (with 3 lags)
  ns = 1 + 1 + n_y + 4 + 4*n_y ;      % size of state
  
  F = zeros(ns,ns);
  ii = 1;
  F(ii+1,ii+1) = 1;  %(tau)
  ii = ii+1;
  for j = 1:n_y;
    F(ii+1,ii+1) = 1;  %(tau)
    ii = ii+1;
  end;
  F(ii+1,ii+1:ii+3) = -ones(1,3);
  F(ii+2:ii+4,ii+1:ii+3) = eye(3);
  ii = ii+4;
  for j = 1:n_y;
    F(ii+1,ii+1:ii+3) = -ones(1,3);
    F(ii+2:ii+4,ii+1:ii+3) = eye(3);
    ii = ii+4;
  end;
  
  G = zeros(ns,1+1+n_y+1+n_y);
  G(1:n_y+2,1:n_y+2) = eye(n_y+2);
  ii = n_y+2;
  jj = n_y+2;
  G(ii+1,jj+1) = 1;
  ii = ii+4; 
  jj = jj+1;
  for j = 1:n_y;
   G(ii+1,jj+1) = 1;
   ii = ii+4; 
   jj = jj+1;
  end;
  
  H = zeros(ns,n_y);
  H(3:n_y+2,1:n_y) = eye(n_y);
  ii = 2+n_y+4;
  for j = 1:n_y;
   H(ii+1,j) = 1;
   ii = ii+4; 
  end;
  H(1,:) = alpha_eps';
  H(2,:) = alpha_tau';
  ii = n_y+2;
  H(ii+1,:) = alpha_s';
  
  Ht = H;
  QQ= diag([(sigma_eps_common.^2);(sigma_dtau_common.^2);(sigma_dtau_unique.^2);(sigma_ds_common.^2);(sigma_ds_unique.^2)]);
  Q_t = G*QQ*G';
  Rt = diag((sigma_eps_unique.^2));
  
  % Set up KF to run
  % Initial conditions
  X1_init = zeros(ns,1);
  P1_init = zeros(ns,ns);
  % All common terms have zero initial conditions; unique taus have vague
  % initial conditions, as are seasonal uniquenesses (with sums to zero); 
  P1_init(3:n_y+2,3:n_y+2) = big*eye(n_y);  % Vague prior for tau_unique initial values 
  % Impose Constraint that s(0)+s(-1)+s(-2)+s(-3) = 0 (so mean is in tau compnents for seasu)
  A = eye(4)-ones(4,4)/4;
  P1_seas=big*A*A';  % Vague prior for seasonal pattern (but sum to zero)
  ii = n_y+2+4;
  for j = 1:n_y;
    P1_init(ii+1:ii+4,ii+1:ii+4) = P1_seas;
    ii = ii+4;
  end;
  X1 = X1_init;
  P1 = P1_init;
  X1t=zeros(ns,nobs+1);
  P1t=zeros(ns,ns,nobs+1);
  X2t=zeros(ns,nobs+1);
  P2t=zeros(ns,ns,nobs+1);
  X1t(:,1)=X1; 
  P1t(:,:,1)=P1;
  X3t=zeros(ns,nobs+1);
  
  for t = 1:nobs;
    yt = y(t,:)';
    [X1,P1,X2,P2] = kfilt(yt,X1,P1,Ht,F,Rt,Q_t);
    X1t(:,t+1)=X1;
    P1t(:,:,t+1)=P1;
    X2t(:,t+1)=X2;
    P2t(:,:,t+1)=P2;
   end;
   X3t(:,nobs+1) = X1;
   P3 = P1;
   X3 = X1;
   for t = nobs:-1:1;
      X1 = X1t(:,t);
      X2 = X2t(:,t+1);
      P1 = P1t(:,:,t);
      P2 = P2t(:,:,t+1);
      tmp = F*P1;
      [AS,R]=linsolve(P2,tmp);
      opts.POSDEF=true;
      if R < 1.0e-12;;
         P2i=pinv(P2);
         AS = P2i*tmp;
      end;
      P3=P1+AS'*(P3-P2)*AS;
      P3 = 0.5*(P3+P3');
      X3=X1+AS'*(X3-X2);
      X3t(:,t) = X3;
   end;
   
   % Compute trend estimate for each sector
   tau_s_common = X3t(2,2:end)';
   tau_s_unique = X3t(3:2+n_y,2:end)';
   tau_f_common = X1t(2,2:end)';
   tau_f_unique = X1t(3:2+n_y,2:end)';
   tau_s_sector = tau_s_common*alpha_tau'+tau_s_unique;
   tau_f_sector = tau_f_common*alpha_tau'+tau_f_unique;
   tau_s = tau_s_sector*share;
   tau_f = tau_f_sector*share;
   
end