function [eps_common,tau_a_common,tau_a_unique,tau_f_common,tau_f_unique,s_a_common,s_a_unique,s_f_common,s_f_unique] = mdraw_tau_seas(y,alpha_eps,alpha_tau,alpha_s,sigma_eps_common,sigma_dtau_common,sigma_s_common,sigma_eps_unique,sigma_dtau_unique,sigma_s_unique);
% 
%       y: T x 1
%       sigma_dtau: T x 1
%       sigma_eps: T x 1
%       
%       Output
%       tau_a: (T+1)x1 -- tau draw .. (t = 0 through T)
%       tau_f: Tx1 -- tau_filtered draw (t = 1 through T)
      
small=1.0e-06;
big = 1.0e+6;
n_y = size(y,2);
nobs = size(y,1);

% Set up State Vector
  % --- State Vector
  %     (1) eps(t)
  %     (2) tau(t)
  %     (3) tau_u(t)
  %     (4) seas(t) (with 3 lags)
  %     (5) seas_u(t) (with 3 lags)
  ns = 1 + 1 + n_y + 4 + 4*n_y ;      % size of state
  
  F = zeros(ns,ns);
  ii = 1;
  F(ii+1,ii+1) = 1;  %(tau)
  ii = ii+1;
  for j = 1:n_y;
    F(ii+1,ii+1) = 1;  %(tau)
    ii = ii+1;
  end;
  F(ii+1,ii+1:ii+3) = -ones(1,3);
  F(ii+2:ii+4,ii+1:ii+3) = eye(3);
  ii = ii+4;
  for j = 1:n_y;
    F(ii+1,ii+1:ii+3) = -ones(1,3);
    F(ii+2:ii+4,ii+1:ii+3) = eye(3);
    ii = ii+4;
  end;
  
  G = zeros(ns,1+1+n_y+1+n_y);
  G(1:n_y+2,1:n_y+2) = eye(n_y+2);
  ii = n_y+2;
  jj = n_y+2;
  G(ii+1,jj+1) = 1;
  ii = ii+4; 
  jj = jj+1;
  for j = 1:n_y;
   G(ii+1,jj+1) = 1;
   ii = ii+4; 
   jj = jj+1;
  end;
  
  H = zeros(ns,n_y);
  H(3:n_y+2,1:n_y) = eye(n_y);
  ii = 2+n_y+4;
  for j = 1:n_y;
   H(ii+1,j) = 1;
   ii = ii+4; 
  end;
  H(1,:) = alpha_eps';
  H(2,:) = alpha_tau';
  ii = n_y+2;
  H(ii+1,:) = alpha_s';
  
  % Set up KF to run
  % Initial conditions
  X1_init = zeros(ns,1);
  P1_init = zeros(ns,ns);
  % All common terms have zero initial conditions; unique taus have vague
  % initial conditions, as are seasonal uniquenesses (with sums to zero); 
  P1_init(3:n_y+2,3:n_y+2) = big*eye(n_y);  % Vague prior for tau_unique initial values 
  % Impose Constraint that s(0)+s(-1)+s(-2)+s(-3) = 0 (so mean is in tau compnents for seasu)
  A = eye(4)-ones(4,4)/4;
  P1_seas=big*A*A';  % Vague prior for seasonal pattern (but sum to zero)
  ii = n_y+2+4;
  for j = 1:n_y;
    P1_init(ii+1:ii+4,ii+1:ii+4) = P1_seas;
    ii = ii+4;
  end;
  X1 = X1_init;
  P1 = P1_init;
  X1t=zeros(ns,nobs+1);
  P1t=zeros(ns,ns,nobs+1);
  X2t=zeros(ns,nobs+1);
  P2t=zeros(ns,ns,nobs+1);
  X1t(:,1)=X1; 
  P1t(:,:,1)=P1;
  X_Draw_Filt = NaN*zeros(nobs+1,ns);  % Draw from filtered .. these are marginal draws, not joint (same as ucsv) ;
  
  % Check for missing values
  if sum(sum(isnan(y))) == 0;
  for t = 1:nobs;
    Ht = H;
    QQ= diag([(sigma_eps_common(t,1).^2) (sigma_dtau_common(t,1).^2) (sigma_dtau_unique(t,:).^2) (sigma_s_common(t,1).^2) (sigma_s_unique(t,:).^2)]');
    Q_t = G*QQ*G';
    Rt = diag((sigma_eps_unique(t,:).^2)');
    yt = y(t,:)';
    [X1,P1,X2,P2] = kfilt(yt,X1,P1,Ht,F,Rt,Q_t);
    X1t(:,t+1)=X1;
    P1t(:,:,t+1)=P1;
    X2t(:,t+1)=X2;
    P2t(:,:,t+1)=P2;
    P1s = sqrt_psd(P1);
    X = X1+P1s*randn(ns,1);
    X_Draw_Filt(t+1,:)=X';
   end;
  
  else;
   for t = 1:nobs;
     Ht = H;
     QQ= diag([(sigma_eps_common(t,1).^2) (sigma_dtau_common(t,1).^2) (sigma_dtau_unique(t,:).^2) (sigma_s_common(t,1).^2) (sigma_s_unique(t,:).^2)]');
     Q_t = G*QQ*G';
     Rt = diag((sigma_eps_unique(t,:).^2)');
     tmp = y(t,:)';
     inan = isnan(tmp);
     yt = tmp(inan==0);
     Ht = Ht(:,inan==0);
     Rt = R_t(inan==0,inan==0);
     [X1,P1,X2,P2] = kfilt(yt,X1,P1,Ht,F,Rt,Q_t);
     X1t(:,t+1)=X1;
     P1t(:,:,t+1)=P1;
     X2t(:,t+1)=X2;
     P2t(:,:,t+1)=P2;
     chol_P1 = chol(P1);
     X = X1+chol_P1'*randn(ns,1);
     X_Draw_Filt(t+1,:)=X';
   end;
 
 end; 
   
   % Draw From State
   X_Draw = NaN*zeros(nobs+1,ns);
   % Initial Draw
   P3 = P1;
   X3 = X1;
   P3s = sqrt_psd(P3);
   X = X3+P3s*randn(ns,1);
   X_Draw(nobs+1,:)=X';
   for t = nobs:-1:1;
      X1 = X1t(:,t);
      X2 = X2t(:,t+1);
      P1 = P1t(:,:,t);
      P2 = P2t(:,:,t+1);
      F_P1 = F*P1;
      [AS,rcond]=linsolve(P2,F_P1);
      if rcond < 1.0e-12;;
           P2i=pinv(P2);
           AS = P2i*F_P1;
      end;
      P3=P1-AS'*F_P1;   % Note .. this is NOT Kalman Smoother .. here we are working back recursively
      P3 = 0.5*(P3+P3');
      X3=X1+AS'*(X-X2);
      X = X3;
      P3s = sqrt_psd(P3);   
      X = X + P3s*randn(ns,1);           
      X_Draw(t,:)=X';
   end;
   eps_common = X_Draw(2:end,1);
   tau_a_common = X_Draw(:,2);
   tau_a_unique = X_Draw(:,3:n_y+2);
   tau_f_common = X_Draw_Filt(2:end,2);
   tau_f_unique = X_Draw_Filt(2:end,3:n_y+2);
   ii = n_y+2;
   s_a_common = [X_Draw(1,ii+3);X_Draw(1,ii+2);X_Draw(:,ii+1)];
   s_a_unique = NaN(size(s_a_common,1),n_y);
   ii = ii+4;
   for j = 1:n_y;
       s_a_unique(:,j) = [X_Draw(1,ii+3);X_Draw(1,ii+2);X_Draw(:,ii+1)];
       ii = ii+4;
   end;
   ii = n_y+2;
   s_f_common = X_Draw_Filt(2:end,ii+1);
   s_f_unique = NaN(size(s_f_common,1),n_y);
   ii = ii+4;
   for j = 1:n_y;
       s_f_unique(:,j) = X_Draw_Filt(2:end,ii+1);
       ii = ii+4;
   end;
   
end