% mucsv_dp_seas_4c
% Compute Implied Weights for smoothed value of trend from UCSV Model
% Plot Values
%
clear all;
small = 1.0e-10;
big = 1.0e+6;                  
rng(663436);

p1 = path;
path(p1,'../m_utilities');

% -- File Directories  
  outdir = 'out/';
  figdir = 'fig/';
  matdir = 'mat/';
%     
  % -- Read in Data --- 
  load_data = 1;  % 1 if reloading data from Excel, etc 
  hicp_data_13_sectors; 
  share_avg = w_disagg;
  % Data Series Used
  % Rearrange disaggregate data so order is igoods,services,food,energy
  mlabel = '_mucsv_tvp_13c_Q0';  
  nsec = size(dp_disagg,2);

  % Sample Period
  first_date = [2001 2];
  last_date = calds(end,:);
  nper = 4;
  ismpl = smpl(calvec,first_date,last_date,nper);
  calvec_ismpl = calvec(ismpl==1);
  notim = size(calvec_ismpl,1);
  str_tmp = [matdir 'calvec_ismpl' mlabel]; save(str_tmp,'calvec_ismpl');
  share_avg = share_avg(ismpl==1,:);
 
  % Read in System parameters
  str_tmp = [matdir 'alpha_eps_mean_pct' mlabel]; load(str_tmp);
  str_tmp = [matdir 'alpha_tau_mean_pct' mlabel]; load(str_tmp);
  str_tmp = [matdir 'alpha_s_mean_pct' mlabel]; load(str_tmp);
  str_tmp = [matdir 'sigma_eps_unique_mean_pct' mlabel]; load(str_tmp);
  str_tmp = [matdir 'sigma_dtau_unique_mean_pct' mlabel]; load(str_tmp);
  str_tmp = [matdir 'sigma_ds_unique_mean_pct' mlabel]; load(str_tmp);
  str_tmp = [matdir 'sigma_eps_common_mean_pct' mlabel]; load(str_tmp);
  str_tmp = [matdir 'sigma_dtau_common_mean_pct' mlabel]; load(str_tmp);
  str_tmp = [matdir 'sigma_ds_common_mean_pct' mlabel]; load(str_tmp);
  str_tmp = [matdir 'scale_eps_unique_mean_pct' mlabel]; load(str_tmp);
  str_tmp = [matdir 'scale_eps_common_mean_pct' mlabel]; load(str_tmp);
  
  alpha_eps = alpha_eps_mean_pct(:,1);
  alpha_tau = alpha_tau_mean_pct(:,1);
  alpha_s = alpha_s_mean_pct(:,1);
  sigma_eps_common = sigma_eps_common_mean_pct(:,1);
  sigma_ds_common = sigma_ds_common_mean_pct(:,1);
  sigma_dtau_common = sigma_dtau_common_mean_pct(:,1);
  sigma_eps_unique = squeeze(sigma_eps_unique_mean_pct(:,1,:));
  sigma_ds_unique = squeeze(sigma_ds_unique_mean_pct(:,1,:));
  sigma_dtau_unique = squeeze(sigma_dtau_unique_mean_pct(:,1,:));
 
  % Use time invariant model .. average values of parameters
  sigma_eps_common = mean(sigma_eps_common);
  sigma_dtau_common = mean(sigma_dtau_common);
  sigma_ds_common = mean(sigma_ds_common);
  sigma_eps_unique = mean(sigma_eps_unique)';
  sigma_ds_unique = mean(sigma_ds_unique)';
  sigma_dtau_unique = mean(sigma_dtau_unique)';
  share = mean(share_avg)';
  
  % Construct some weights
  nn = 40;  % Weights +or- nn of date
  
  T = 500;
  Tmid = floor(T/2);
  wght_mat_s = NaN(2*nn+1,nsec);  % Weight for agg_tau(t) by y(t,i) -- smoothed
  wght_mat_f = NaN(2*nn+1,nsec);  % Weight for agg_tau(t) by y(t,i) -- filtered
  
  for i = 1:nsec;
      for t = -nn:nn;
          y = zeros(T,nsec);
          y(Tmid+t,i) = 1;
          [tau_a,tau_f] = m_agg_tau_weight(y,alpha_eps,alpha_tau,alpha_s,sigma_eps_common,sigma_ds_common,sigma_dtau_common,sigma_eps_unique,sigma_ds_unique,sigma_dtau_unique,share);
          wght_mat_s(nn+1+t,i) = tau_a(Tmid);
          wght_mat_f(nn+1+t,i) = tau_f(Tmid);
      end;
  end;
  
  wght_mat_s = wght_mat_s./repmat(share',2*nn+1,1);
  wght_mat_f = wght_mat_f./repmat(share',2*nn+1,1);
  
  str_tmp = [matdir 'wght_mat_s' mlabel]; save(str_tmp,'wght_mat_s');
  str_tmp = [matdir 'wght_mat_f' mlabel]; save(str_tmp,'wght_mat_f');
 
  path(p1);
  
  

  