function [tau_s,tau_f,s_s] = m_agg_tau_weight(y,var_eps,var_dtau,var_ds)
      
small=1.0e-6;
big = 1.0e+6;
nobs=size(y,1);

% Set up State Vector
  % --- State Vector
  %     (1) tau(t)
  %     (2) s(t)
  %     (3) s(t-1)
  %     (4) s(t-2)
  %     (5) s(t-3)  ... add additional lag to get initial condition right
  ns = 5;      % size of state
  Q = zeros(ns,ns);
  F = zeros(ns,ns);
  F(1,1) = 1;
  F(2,2:4) = -ones(1,3);
  F(3:5,2:4) = eye(3);
  H = zeros(ns,1);
  H(1,1) = 1;
  H(2,1) = 1;
  H_t = H;
  Q_t = Q;
  Q_t(1,1) = var_dtau;
  Q_t(2,2) = var_ds;
  R_t = var_eps;
  
 % Set up KF to run
  % Initial conditions
  X1_init = zeros(ns,1);
  P1_init = zeros(ns,ns);
  P1_init(1,1) = big;
  % Impose Constraint that s(0)+s(-1)+s(-2)+s(-3) = 0 (so mean is in tau compnents)
  A = eye(4)-ones(4,4)/4;
  P1_init(2:5,2:5) = big*A*A';  % Vague prior for seasonal pattern (but sum to zero)
  X1 = X1_init;
  P1 = P1_init;
  X1t=zeros(ns,nobs+1);
  P1t=zeros(ns,ns,nobs+1);
  X2t=zeros(ns,nobs+1);
  P2t=zeros(ns,ns,nobs+1);
  X1t(:,1)=X1; 
  P1t(:,:,1)=P1;
  X3t=zeros(ns,nobs+1);
 
  for t = 1:nobs;
    yt = y(t);
    [X1,P1,X2,P2] = kfilt(yt,X1,P1,H_t,F,R_t,Q_t);
    X1t(:,t+1)=X1;
    P1t(:,:,t+1)=P1;
    X2t(:,t+1)=X2;
    P2t(:,:,t+1)=P2;
   end;
   X3t(:,nobs+1) = X1;
   P3 = P1;
   X3 = X1;
   for t = nobs:-1:1;
      X1 = X1t(:,t);
      X2 = X2t(:,t+1);
      P1 = P1t(:,:,t);
      P2 = P2t(:,:,t+1);
      tmp = F*P1;
      [AS,R]=linsolve(P2,tmp);
      opts.POSDEF=true;
      if R < 1.0e-12;;
         P2i=pinv(P2);
         AS = P2i*tmp;
      end;
      P3=P1+AS'*(P3-P2)*AS;
      P3 = 0.5*(P3+P3');
      X3=X1+AS'*(X3-X2);
      X3t(:,t) = X3;
   end;
   
   % Compute trend estimate for each sector
   tau_s = X3t(1,2:end)';
   tau_f = X1t(1,2:end)';
   s_s = X3t(2,2:end)';
   
end