% ucsv_dp_seas.m -- Univariate UCSV model Allowing for outlier in eps and seasonal factor
% 1/22/2019
%

clear all;
small = 1.0e-10;
big = 1.0e+6;                  
rng(663436);

p1 = path;
path(p1,'../m_utilities');
 
  % -- File Directories  
  outdir = 'out/';
  figdir = 'fig/';
  matdir = 'mat/';
%     
  % -- Read in Data --- 
  load_data = 1;  % 1 if reloading data from Excel, etc 
  hicp_data_13_sectors;  
  % Data Series Used
  dp_agg = dp_agg;
  outlabel = 'dp_agg';  
  namevec = {'dp_agg'};
  dp = dp_agg;
  
  % Dates
  first_date = [2001 2];
  last_date = calds(end,:);
  nper = 4;
  
  ismpl = smpl(calvec,first_date,last_date,nper);
  calvec_ismpl = calvec(ismpl==1);
  dnobs_ismpl = size(calvec_ismpl,1);
  str_tmp = [matdir 'calvec_ismpl_ucsv']; save(str_tmp,'calvec_ismpl');
  
  % Parameters for UCSV Draws
  n_burnin = 10000;     % Discarded Draws
  %n_burnin = 100;     % Discarded Draws
  n_draws_save = 5000;  % Number of Draws to Save
  %n_draws_save = 50;  % Number of Draws to Save
  k_draws = 10;         % Save results every k_draws
  n_draws =  n_draws_save*k_draws;   % Total Number of Draws after burnin
  
  % Parameters for scale mixture of epsilon component
  scl_eps_vec = [1; linspace(2.0,10.0,9)'];
  ps_mean = 1 - 1/(4*nper);              % Outlier every 4 years
  ps_prior_obs = nper*10;                % Sample size of 10 years for prior
  ps_prior_a = ps_mean*ps_prior_obs;     % "alpha" in beta prior
  ps_prior_b = (1-ps_mean)*ps_prior_obs; % "beta" in beta prior
  ps_prior = [ps_prior_a ps_prior_b];
  
  % -- Parameters for RW Innovation Variance -- 
  % Gvalues .. for evolution of standard deviations over a year (nper periods)
  ng = 5;      % Number of grid points for approximate uniform prior
  g_dtau_min = 0.001;
  g_dtau_max = 0.20;
  g_dtau_values = linspace(g_dtau_min,g_dtau_max,ng)';
  p_g_dtau_values = ones(ng,1)/ng;
  g_eps_min = 0.001;
  g_eps_max = 0.20;
  g_eps_values = linspace(g_eps_min,g_eps_max,ng)';
  p_g_eps_values = ones(ng,1)/ng;
  % Convert to standard deviation per period
  g_dtau_values = g_dtau_values/sqrt(nper);
  g_eps_values = g_eps_values/sqrt(nper);
  % Convert to g-values for variances instead of standard deviations (ln(s^2) = 2*ln(s))
  g_dtau_values=2*g_dtau_values;
  g_eps_values=2*g_eps_values;
  % Save priors
  g_eps_prior = [g_eps_values p_g_eps_values];
  g_dtau_prior = [g_dtau_values p_g_dtau_values];
  g_s_prior = g_dtau_prior;  % Same g-prior for seasonal shocks
   
  tmp = char(namevec(1));
  ulabel = [tmp '_' outlabel]; 
  fprintf(['Carrying out Calculations for ' ulabel '\n']);
    
  y = dp(ismpl==1,1);
  [tau_draws,tau_f_draws,s_draws,s_f_draws,sigma_dtau_draws,sigma_eps_draws,sigma_s_draws,g_eps_draws,g_dtau_draws,g_s_draws,scl_eps_draws,sigmatotal_eps_draws,ps_draws] = ucsv_seas_outlier(y,n_burnin,n_draws,k_draws,g_eps_prior,g_dtau_prior,g_s_prior,scl_eps_vec,ps_prior);
  
  % Compute Posterior for g's
  g_eps_prior_post = g_summary(g_eps_prior,g_eps_draws);
  g_dtau_prior_post = g_summary(g_dtau_prior,g_dtau_draws);
  g_s_prior_post = g_summary(g_s_prior,g_s_draws);
  str_tmp = [matdir 'g_eps_prior_post' ulabel]; save(str_tmp,'g_eps_prior_post');
  str_tmp = [matdir 'g_dtau_prior_post' ulabel]; save(str_tmp,'g_dtau_prior_post');
  str_tmp = [matdir 'g_s_prior_post' ulabel]; save(str_tmp,'g_s_prior_post');
  
  % Compute posterior for variances
  pctvec = [0.05 1/6 0.50 5/6 0.95]';
  tmp = sigma_dtau_draws.^2;
  var_dtau_mean_pct = post_mean_pct(tmp',pctvec);
  tmp = sigma_eps_draws.^2;
  var_eps_mean_pct = post_mean_pct(tmp',pctvec);
  tmp = sigmatotal_eps_draws.^2;
  vartotal_eps_mean_pct = post_mean_pct(tmp',pctvec);
  tmp = sigma_s_draws.^2;
  var_s_mean_pct = post_mean_pct(tmp',pctvec);
  str_tmp = [matdir 'var_eps_mean_pct' ulabel]; save(str_tmp,'var_eps_mean_pct');
  str_tmp = [matdir 'var_dtau_mean_pct' ulabel]; save(str_tmp,'var_dtau_mean_pct');
  str_tmp = [matdir 'var_s_mean_pct' ulabel]; save(str_tmp,'var_s_mean_pct');
  str_tmp = [matdir 'vartotal_eps_mean_pct' ulabel]; save(str_tmp,'vartotal_eps_mean_pct');
  
  sigma_dtau_mean_pct = post_mean_pct(sigma_dtau_draws',pctvec);
  sigma_s_mean_pct = post_mean_pct(sigma_s_draws',pctvec);
  sigma_eps_mean_pct = post_mean_pct(sigma_eps_draws',pctvec);
  sigmatotal_eps_mean_pct = post_mean_pct(sigmatotal_eps_draws',pctvec);
  str_tmp = [matdir 'sigma_eps_mean_pct' ulabel]; save(str_tmp,'sigma_eps_mean_pct');
  str_tmp = [matdir 'sigma_dtau_mean_pct' ulabel]; save(str_tmp,'sigma_dtau_mean_pct');
  str_tmp = [matdir 'sigma_s_mean_pct' ulabel]; save(str_tmp,'sigma_s_mean_pct');
  str_tmp = [matdir 'sigmatotal_eps_mean_pct' ulabel]; save(str_tmp,'sigmatotal_eps_mean_pct');
  
  % Compute posteriors for trend values
  tau_mean_pct = post_mean_pct(tau_draws',pctvec);
  tau_f_mean_pct = post_mean_pct(tau_f_draws',pctvec);
  str_tmp = [matdir 'tau_mean_pct' ulabel]; save(str_tmp,'tau_mean_pct');
  str_tmp = [matdir 'tau_f_mean_pct' ulabel]; save(str_tmp,'tau_f_mean_pct');
  tau_f_mean = mean(tau_f_draws,2);
  tmp = mean(tau_f_draws.^2,2);
  tau_f_var = tmp-tau_f_mean.^2;
  tau_f_sd = sqrt(tau_f_var);
  tau_f_mean_sd = [tau_f_mean tau_f_sd];
  str_tmp = [matdir 'tau_f_mean_sd' ulabel]; save(str_tmp,'tau_f_mean_sd');
  
  s_mean_pct = post_mean_pct(s_draws',pctvec);
  str_tmp = [matdir 's_mean_pct' ulabel]; save(str_tmp,'s_mean_pct');
  s_f_mean_pct = post_mean_pct(s_f_draws',pctvec);
  str_tmp = [matdir 's_f_mean_pct' ulabel]; save(str_tmp,'s_f_mean_pct');
   
  tau_mean = mean(tau_draws,2);
  tmp = mean(tau_draws.^2,2);
  tau_var = tmp-tau_mean.^2;
  tau_sd = sqrt(tau_var);
  tau_mean_sd = [tau_mean tau_sd];
  str_tmp = [matdir 'tau_mean_sd' ulabel]; save(str_tmp,'tau_mean_sd');
 
  % Compute posteriors for scale_eps_values
  pctvec = [1/6 0.50 5/6]';
  scl_eps_pct = post_mean_pct(scl_eps_draws',pctvec);
  str_tmp = [matdir 'scl_eps_pct' ulabel]; save(str_tmp,'scl_eps_pct');
    
  % Compute Posterior for ps's
  ps_mean_pct = post_mean_pct(ps_draws,pctvec);
  str_tmp = [matdir 'ps_mean_pct' ulabel]; save(str_tmp,'ps_mean_pct');
  
  path(p1);
  