% ucsv_dp_seas_weights
% Compute Implied Weights for smoothed value of trend from UCSV Model
%
clear all;
small = 1.0e-10;
big = 1.0e+6;                  
rng(663436);

% -- File Directories  
  outdir = 'out/';
  figdir = 'fig/';
  matdir = 'mat/';
%     
  % -- Read in Data --- 
  load_data = 1;  % 1 if reloading data from Excel, etc 
  hicp_data_13_sectors;  
  outlabel = 'dp_agg';  
  namevec = {'dp_agg'};
  tmp = char(namevec(1));
  ulabel = [tmp '_' outlabel]; 
   
  % Read in System parameters
  str_tmp = [matdir 'sigma_eps_mean_pct' ulabel]; load(str_tmp);
  str_tmp = [matdir 'sigma_dtau_mean_pct' ulabel]; load(str_tmp);
  str_tmp = [matdir 'sigma_s_mean_pct' ulabel]; load(str_tmp);
  
  sigma_eps = sigma_eps_mean_pct(:,1);
  sigma_dtau = sigma_dtau_mean_pct(:,1);
  sigma_ds = sigma_s_mean_pct(:,1);
  
  % Use time invariant model .. average values of parameters
  sigma_eps = mean(sigma_eps);
  sigma_dtau = mean(sigma_dtau);
  sigma_ds = mean(sigma_ds);
  var_eps = sigma_eps^2;
  var_dtau = sigma_dtau^2;
  var_ds = sigma_ds^2;
  
  % Construct some weights
  nn = 40;  % Weights +or- nn of date
  
  T = 500;
  Tmid = floor(T/2);
  wght_mat_s = NaN(2*nn+1,1);  % Weight for agg_tau(t) by y(t,i) -- smoothed
  wght_mat_f = NaN(2*nn+1,1);  % Weight for agg_tau(t) by y(t,i) -- filtered
  wght_mat_s_s = NaN(2*nn+1,1);  % Weight for agg_s(t) by y(t,i) -- smoothed
  
  for t = -nn:nn;
          y = zeros(T,1);
          y(Tmid+t) = 1;
          [tau_s,tau_f,s_s] = u_agg_tau_weight(y,var_eps,var_dtau,var_ds);
          wght_mat_s(nn+1+t) = tau_s(Tmid);
          wght_mat_f(nn+1+t) = tau_f(Tmid);
          wght_mat_s_s(nn+1+t) = s_s(Tmid);
  end;
  
  str_tmp = [matdir 'wght_mat_s' ulabel]; save(str_tmp,'wght_mat_s');
  str_tmp = [matdir 'wght_mat_f' ulabel]; save(str_tmp,'wght_mat_f');
  str_tmp = [matdir 'wght_mat_s_s' ulabel]; save(str_tmp,'wght_mat_s_s');
  
%   X = (-nn:1:nn)';
%   plot(X,wght_mat_s);
%  
  
  

  